@@ -1,8 +1,10 @@
 from collections import namedtuple
 from typing import List, Union, Optional
 
+import dask
 from dask.dataframe import DataFrame
 from dask import dataframe as dd, delayed
+from dask.delayed import Delayed
 from fsspec.core import OpenFile, open_files
 from pandas import DataFrame as PDataFrame
 from pyexcelerate import Workbook
@@ -36,7 +38,7 @@ class FormatJob:
             "partition_columns": [Use(str)],
             "filter_columns": [Use(str)],
             SOptional("line_terminator", default="\n"): str,
-            SOptional("partitions", default="auto"): str
+            SOptional("partitions"): Use(int)
         }
         return Schema(schema)
 
@@ -58,14 +60,19 @@ class ValidFormatJob:
         self.partition_columns: List[str] = t.partition_columns
         self.filter_columns: List[str] = t.filter_columns
         self.line_terminator: str = t.line_terminator
-        self.partitions: str = t.partitions
+        self.partitions: Optional[int]
+        try:
+            self.partitions = t.partitions
+        except AttributeError:
+            self.partitions = None 
+            
         
 
     def df(self) -> Union[DataFrame, InvalidJob]:
         paths = self.input_paths
         df: DataFrame
         if self.input_format in VALID_TEXT_FORMATS:
-            df = dd.read_csv(paths, lineterminator=self.line_terminator)
+            df = dd.read_csv(paths, lineterminator=self.line_terminator, assume_missing=True, sample=25000000)
             final = df
         elif self.input_format == "parquet":
             df = dd.read_parquet(paths)
@@ -142,6 +149,10 @@ class ValidFormatJob:
             final = InvalidJob(f"Invalid input format: {self.input_format}")
 
         return final
+    
+    def df_to_simple(self, df: DataFrame, output_path: str) -> List[Delayed]:
+        a: List[Delayed] = dd.to_csv(df, output_path, index=False)
+        return a
 
     def check_columns(self, df: DataFrame, columns: List[str]) -> Union[bool, InvalidJob]:
         keys = df.dtypes.keys()
@@ -151,34 +162,34 @@ class ValidFormatJob:
         else:
             return InvalidJob(f"Filter columns {', '.join(diff)} not a subset of {', '.join(keys)}")
 
-    def repartition(self, df: DataFrame, cluster_spec: ClusterSpec, num_partitions: str) -> Union[DataFrame, InvalidJob]:
-
+    def repartition(self, df: DataFrame, cluster_spec: ClusterSpec, partitions: Optional[int]) -> Union[DataFrame, InvalidJob]:
         size = df.size.compute()
-        workers = cluster_spec.num_workers() or self.partitions
         final: Union[DataFrame, InvalidJob]
-        if num_partitions == "auto":
-            parts = workers
-        else:
-            try:
-                parts = int(num_partitions)
-            except ValueError as e:
-                parts = InvalidJob(f"Invalid partitions specification: {num_partitions}")
+        
+        num_partitions = self.partitions
 
-        if not isinstance(parts, InvalidJob):
-            even_worker_partition_size = size / workers
-            max_partition_size = cluster_spec.max_partition_size()
-            if even_worker_partition_size > max_partition_size:
-                final = InvalidJob(
-                    f"Partitions too large for workers: (size, max) = ({even_worker_partition_size}, {max_partition_size}).  Add more workers or increase worker memory.")
-            else:
-                final = df.repartition(partition_size=even_worker_partition_size).repartition(npartitions=parts)
+        if cluster_spec.valid():
+            cluster_spec_num_workers = cluster_spec.num_workers() 
         else:
-            final = parts
+            cluster_spec_num_workers = None
+            
+        num_workers = num_partitions or cluster_spec_num_workers 
+        
+        if num_workers:
+            max_partition_size = cluster_spec.max_partition_size()
+            even_partition_size = size / num_workers 
+            if even_partition_size > max_partition_size:
+                final = InvalidJob(f"Partitions too large for workers: (size, max) = ({even_partition_size}, {max_partition_size}).  Add more workers or increase worker memory.")
+            else:
+                final = df.repartition(partition_size=even_partition_size).repartition(npartitions=num_workers)
+        else:
+            final = df
 
         return final
 
-    def run(self, cluster_spec: ClusterSpec) -> Union[ExecutedJob, InvalidJob]:
+    def run(self, cluster_spec: ClusterSpec):
         df = self.df()
+        
         if isinstance(df, DataFrame):
             def write_partitioned(df: PDataFrame, partition_columns: List[str]):
                 ddf = dd.from_pandas(df, npartitions=cluster_spec.num_workers() or self.partitions)
@@ -212,11 +223,13 @@ class ValidFormatJob:
                     final = ExecutedJob(", ".join(results))
             else:
                 df = self.repartition(df, cluster_spec, self.partitions)
-                final: Union[ExecutedJob, InvalidJob] = self.df_to(df)
-
+                if isinstance(df, DataFrame):
+                    final = self.df_to(df)
+                else:
+                    final = df
 
         else:
             final = df
-
+            
         return final
 
